#ifndef _XPROTO_H_
#define _XPROTO_H_

/** xproto.h
*
*   Include file for External Protocol Handling
*
**/

/*
 * The xpro_option structure
 */
struct xpr_option
{
    char *xpro_description;	/* description of the option                  */
    int xpro_type;		/* type of option                             */
    char *xpro_value;		/* pointer to a buffer with the current value */
    int xpro_length;		/* buffer size                                */
};

typedef struct XPR_UPDATE
{
    int xpru_updatemask;
    char *xpru_protocol;
    char *xpru_filename;
    int xpru_filesize;
    char *xpru_msg;
    char *xpru_errormsg;
    int xpru_blocks;
    int xpru_blocksize;
    int xpru_bytes;
    int xpru_errors;
    int xpru_timeouts;
    int xpru_packettype;
    int xpru_packetdelay;
    int xpru_chardelay;
    char *xpru_blockcheck;
    char *xpru_expecttime;
    char *xpru_elapsedtime;
    int xpru_datarate;
    int xpru_reserved1;
    int xpru_reserved2;
    int xpru_reserved3;
    int xpru_reserved4;
    int xpru_reserved5;
} XPRU, *PXPRU;

/*
 * The structure
 */

typedef struct __xfctl FCTL;
  
typedef struct XPR_IO
{
    char *xpr_filename;
    struct __xfctl * (*xpr_fopen) (char *, char *);
    int (*xpr_fclose) (struct __xfctl **);
    int (*xpr_fread) ( char *,  int,  int,  struct __xfctl *);
    int (*xpr_fwrite) ( char *,  int,  int,  struct __xfctl *);
    unsigned int (*xpr_sread) ( char *,  int, int);
    int (*xpr_swrite) ( char *,  int);
    int (*xpr_sflush) (void);
    int (*xpr_update) (struct XPR_UPDATE *);
    int (*xpr_chkabort) (void);
    int (*xpr_chkmisc) (void);
    int (*xpr_gets) ( char *,  char *);
    int (*xpr_setserial) ( int);
    int (*xpr_ffirst) ( char *,  char *);
    int (*xpr_fnext) ( int,  char *,  char *);
    int (*xpr_finfo) ( char *,  int); 
    int (*xpr_fseek) ( struct __xfctl *,  int,  int);
    int xpr_extension;
    void *xpr_data;
    unsigned int (*xpr_options) ( struct xpr_option *, int);
    int (*xpr_unlink) ( char *);
    int (*xpr_squery) (void);
    int (*xpr_getptr) ( int);
} XIO, *PXIO;

/*
 * Number of defined extensions
 */
#define XPR_EXTENSION 4L

/*
 * Flags returned by XProtocolSetup()
 */
#define XPRS_FAILURE    0x00000000L
#define XPRS_SUCCESS    0x00000001L
#define XPRS_NORECREQ   0x00000002L
#define XPRS_NOSNDREQ   0x00000004L
#define XPRS_HOSTMON    0x00000008L
#define XPRS_USERMON    0x00000010L
#define XPRS_HOSTNOWAIT 0x00000020L

/*
 * The update structure
 */

/*
 * The possible bit values for the xpru_updatemask are:
 */
#define XPRU_PROTOCOL           0x00000001L
#define XPRU_FILENAME           0x00000002L
#define XPRU_FILESIZE           0x00000004L
#define XPRU_MSG                0x00000008L
#define XPRU_ERRORMSG           0x00000010L
#define XPRU_BLOCKS             0x00000020L
#define XPRU_BLOCKSIZE          0x00000040L
#define XPRU_BYTES              0x00000080L
#define XPRU_ERRORS             0x00000100L
#define XPRU_TIMEOUTS           0x00000200L
#define XPRU_PACKETTYPE         0x00000400L
#define XPRU_PACKETDELAY        0x00000800L
#define XPRU_CHARDELAY          0x00001000L
#define XPRU_BLOCKCHECK         0x00002000L
#define XPRU_EXPECTTIME         0x00004000L
#define XPRU_ELAPSEDTIME        0x00008000L
#define XPRU_DATARATE           0x00010000L

/*
 * Valid values for xpro_type are:
 */
#define XPRO_BOOLEAN 1L		/* xpro_value is "yes", "no", "on" or "off"   */
#define XPRO_LONG    2L		/* xpro_value is string representing a number */
#define XPRO_STRING  3L		/* xpro_value is a string                     */
#define XPRO_HEADER  4L		/* xpro_value is ignored                      */
#define XPRO_COMMAND 5L		/* xpro_value is ignored                      */
#define XPRO_COMMPAR 6L		/* xpro_value contains command parameters     */


#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif

extern unsigned short updcrc (unsigned char cp, unsigned short crc);

#endif

